import pandas as pd
import numpy as np
from datetime import datetime, timedelta

# from get_forex_data import get_exchange_rate
from ..courtois.fmp_url import get_data_url
from .get_collection_fs import company_profile, get_fs_some

def get_exchange_rate(apikey: str='apikey', exchange: str='EURUSD', period: str='daily',\
    start_date: str= "2023-10-01", end_date: str='2023-10-04'):
    """
    params:
        apikey -- to access data from FMP;
        exchange -- required conversion from 'EUR' to 'USD';
        period -- either 'min', or 'hour', or 'daily';
        start_date -- the date from which we collecting data, should from 2005-01-01
        end_date -- the date to which we collecting data
    return: list of dictionaries
    """
    if period=='daily':
        url = (f"https://financialmodelingprep.com/api/v3/historical-price-full/{exchange}?from={start_date}&to={end_date}&apikey={apikey}")

    data = get_data_url(url=url)
    return data

def convert_data_to_common_currency(df, apikey: str='apikey', base: str='USD', data: str='finance', skipping=[]):
    """
    params:
        df -- data-frame numerical values of which we want to convert to commone currendy (base = 'USD')
        apikey -- to access data from FMP;
        data -- it can be either 'finance' (data of financial statement usually has data of 'reportedCurrency') or 'stock' 
            (stock data for which we need to inquire currency);
        skipping -- columns that we don't want to convert;
    return: list of dictionaries
    """
    if 'symbol' not in df.columns:
        print("We don't have the ticker data")
        return False
    else:
        tickers_df = set(df['symbol'])

    if 'currency' not in df.columns:
        df['currency'] = None
        for ticker in tickers_df:
            profile = company_profile(apikey=apikey, symbol=ticker)
            if profile != []:
                profile = profile[0]
                if 'currency' not in profile.keys():
                    if data == 'stock':
                        df.loc[df['symbol'] == ticker, 'currency'] = \
                            get_fs_some(apikey=apikey, symbol=ticker, period='quarter', limit=2)[0]['reportedCurrency']
                    elif data == 'finance':
                        if 'reportedCurrency' in df.columns:
                            df.loc[df['symbol'] == ticker, 'currency'] = df[df['symbol'] == ticker]['reportedCurrency'].mode()[0]
                    else:
                        print("We don't have reported currency information for financial data")
                        return False
                else:
                    df.loc[df['symbol'] == ticker, 'currency'] = profile['currency']


    df['currency'] = df['currency'].str.replace('ZAc', 'ZAR')
    df['currency'] = df['currency'].str.replace('ILA', 'ILS')

    if 'reportedCurrency' not in df.columns:
        df['reportedCurrency'] = None

    for i in df.index:
        if (df.loc[i, 'reportedCurrency'] != df.loc[i, 'reportedCurrency']) or (df.loc[i, 'reportedCurrency'] == None):
            df.loc[i, 'reportedCurrency'] = df.loc[i, 'currency']

    df['reportedCurrency'] = df['reportedCurrency'].str.strip()

    exchange_rate = df[['symbol', 'date']]
    exchange_rate.loc[df['reportedCurrency']==base, 'exchange_rate'] = 1

    j=0;
    ex_currendies = {}

    for i in df.index:
        if (df.loc[i, 'reportedCurrency'] == base):
            if (i%30==0):
                print(df.loc[i, 'reportedCurrency']);
        else :
            if (df.loc[i, 'reportedCurrency'], df.loc[i, 'date']) in ex_currendies.keys():
                exchange_rate.loc[i, 'exchange_rate'] = ex_currendies[(df.loc[i, 'reportedCurrency'], df.loc[i, 'date'])]
            else:
                date = datetime.strptime(df.loc[i, 'date'], '%Y-%m-%d');
                start_date = date - timedelta(days=3);
                start_date = start_date.strftime('%Y-%m-%d')
                end_date = date  + timedelta(days=4);
                end_date = end_date.strftime('%Y-%m-%d')
                x = df.loc[i, 'reportedCurrency'] + base;
                data = get_exchange_rate(apikey=apikey, exchange=x, start_date=start_date, end_date=end_date)
                data = pd.DataFrame(data["historical"])
                data['date'] = pd.to_datetime(data['date'])
                data.set_index('date', inplace=True)
                try:
                    exchange_rate.loc[i, 'exchange_rate'] =  np.nanmedian(data.loc[date, 'adjClose'])
                    ex_currendies[(df.loc[i, 'reportedCurrency'], df.loc[i, 'date'])] = exchange_rate.loc[i, 'exchange_rate']
                    print(df.loc[i, 'reportedCurrency'], df.loc[i, 'date'], i, 'try');
                except: 
                    ser = pd.to_numeric(data['adjClose'], downcast='float')
                    m = np.nanmedian(ser)
                    exchange_rate.loc[i, 'exchange_rate'] = m
                    ex_currendies[(df.loc[i, 'reportedCurrency'], df.loc[i, 'date'])] = exchange_rate.loc[i, 'exchange_rate']
                    print(df.loc[i, 'reportedCurrency'], df.loc[i, 'date'], i, 'except', m);

    df_usd = df.copy()

    skipping = list(set(skipping).union(set(['symbol', 'date', 'reportedCurrency', 'sector',\
        'industry', 'numberOfShares', 'currency', 'country'])))

    for col in df[df.columns.difference(skipping)].columns:
        df_usd[col] = df[col].mul(exchange_rate['exchange_rate'])
        # print(col)

    return df_usd


"""Top-level package for fmp_tcc."""

__author__ = """Bayan Saparbayeva, AlexandreBayan Saparbayeva, Alexandre Courtois Auberger, Bayan Saparbayeva, and Vish Patel"""
__email__ = 'saparbayevabt@gmail.com'
__version__ = '0.1.0'



import plotly.express as px

from plotly.subplots import make_subplots
import plotly.graph_objects as go

from matplotlib import colors

def plot_with_margin(data_frame, x: str, y: str, color: str, palette: dict, error: str=None, upper_bound: str=None, lower_bound: str=None, title: str='Continuous'):

    if (type(error)!=str):
        if ((type(upper_bound)!=str) or (type(lower_bound)!=str)):
            print("No error and no bound parameters, you can use px.line")
            return False
    else:
        lower_bound='lower_bound'
        data_frame[lower_bound] = data_frame[y] - data_frame[error]
        upper_bound='upper_bound'
        data_frame[upper_bound] = data_frame[y] + data_frame[error]

    colorss = set(data_frame[color])

    if (colorss != set(palette.keys())):
        print(f"The {color} column and the keys of palette are not same")
        return False

    go_scatters = []

    for colour in colorss:
        if palette[colour][:3]=='rgb':
            fillcolour = 'rgba' + palette[colour][3:-1] + ', 0.3)'
        else:
            # print(palette[colour])
            fillcolour = colors.to_rgb(palette[colour])
            fillcolour = (int(fillcolour[0]), int(fillcolour[1]), int(fillcolour[2]), 0.3)
            fillcolour = 'rgba' + str(fillcolour)
            
        tmp_df = data_frame[data_frame[color]==colour]

        go_scatters.append(
            go.Scatter(
                name=str(colour), x=tmp_df[x], y=tmp_df[y], mode='lines',
                line=dict(color=palette[colour])
            ))
        # go_scatters.append(
        #     go.Scatter(x=tmp_df[x]+tmp_df[x][::-1], # x, then x reversed
        #                y=tmp_df[upper_bound]+tmp_df[lower_bound][::-1], # upper, then lower reversed
        #                fill='toself', fillcolor=fillcolour, line=dict(color='rgba(255,255,255,0)'), showlegend=False
        #     ))
        go_scatters.append(
            go.Scatter(
                name='Upper Bound', x=tmp_df[x], y=tmp_df[upper_bound], mode='lines',
                line=dict(width=0), showlegend=False
            ))
        go_scatters.append(
            go.Scatter(
                name='Lower Bound', x=tmp_df[x], y=tmp_df[lower_bound], mode='lines',
                line=dict(width=0),
                fill='tonexty', showlegend=False
            ))

    fig = go.Figure(go_scatters)
    fig.update_layout(
        yaxis_title=y,
        title=title)
    
    return fig



try:
    # For Python 3.0 and later
    from urllib.request import urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen

import certifi
import json

def get_data_url(url): #Get data using API link - NOT fmpsdk lib
    response = urlopen(url, cafile=certifi.where())
    data = response.read().decode("utf-8")
    data = json.loads(data)
    return data


import plotly.express as px

from plotly.subplots import make_subplots
import plotly.graph_objects as go

def plot_coef_corr_countryGroup(data_frame, palette: dict):
    
    slope = {}
    correlation = {}

    for region in set(data_frame['countryGroup']):
        slope[region] = go.Scatter(x=data_frame[data_frame['countryGroup'] == region]['date'], 
                              y=data_frame[data_frame['countryGroup'] == region]['slope'],
                              name='slope of ' + region, legendgroup='slope of ' + region,
                              line=dict(color=palette[region]))

    
        correlation[region] = go.Scatter(x=data_frame[data_frame['countryGroup']==region]['date'], 
                              y=data_frame[data_frame['countryGroup']==region]['correlation'], 
                              name='correlation of ' + region, 
                              legendgroup='correlation of ' + region,
                              line=dict(color=palette[region]), opacity=0.5)
        
    fig = make_subplots(rows=2, cols=1, row_heights=[0.7, 0.3])

    for key in slope.keys():
        fig.append_trace(slope[key], row=1, col=1)
    for key in correlation.keys():    
        fig.append_trace(correlation[key], row=2, col=1)

    
    return fig

def plot_coef_corr_industry(data_frame, palette):

    slope = {}
    correlation = {}

    for industry in set(data_frame['industry']):
        slope[industry] = go.Scatter(x=data_frame[data_frame['industry']==industry]['date'], 
                              y=data_frame[data_frame['industry']==industry]['slope'],
                              name='slope of ' + industry, legendgroup='slope of ' + industry, 
                              line=dict(color=palette[industry]))

    
        correlation[industry] = go.Scatter(x=data_frame[data_frame['industry']==industry]['date'], 
                              y=data_frame[data_frame['industry']==industry]['correlation'], 
                              name='correlation of ' + industry, 
                              legendgroup='correlation of ' + industry, 
                              line=dict(color=palette[industry]), opacity=0.5)
        
    fig = make_subplots(rows=2, cols=1, row_heights=[0.7, 0.3])

    for key in slope.keys():
        fig.append_trace(slope[key], row=1, col=1)
    for key in correlation.keys():    
        fig.append_trace(correlation[key], row=2, col=1)
    
    return fig

def camel_case_split(s):
    # use map to add an underscore before each uppercase letter
    modified_string = list(map(lambda x: ' ' + x if x.isupper() else x, s))
    modified_string[0] = modified_string[0].upper()
    split_string = ''.join(modified_string)

    return split_string